﻿using System;
using System.Collections.Generic;
using System.Windows.Controls;
using System.Net;
using System.Reflection;
using System.IO;
using System.Windows;
using System.Windows.Resources;
using System.Windows.Markup;



namespace Samples
{
    public class DownloaderData
    {
        public string XapName { get; set; }
        public string AssemblyName { get; set; }
        public string ClassName { get; set; }
        public PackageContent ActionRequired { get; set; }
    }


    public enum PackageContent
    {
        ClassFromAssembly = 1,
        ResourceFromAssembly = 2
    }

    public class Downloader : IDownloader
    {
        #region Local members
        protected DownloaderData m_data; 
        #endregion

        #region IDownloader Members

        public event EventHandler<XapEventArgs> XapDownloaded;


        public void LoadAssemblyFromPackage(string xap, string asm, string cls)
        {
            Initialize(xap, asm, cls, PackageContent.ClassFromAssembly);

            Stream bits = LookupCacheForPackage();
            if (bits == null)
                StartDownload();
            else
            {
                // Process and extract resources
                FindClassFromAssembly(bits);
            }
        }

        #endregion

        #region Helpers

        private Stream LookupCacheForPackage()
        {
            // Look up the XAP package for the assembly 
            string xapFile = m_data.XapName;

            return DownloadCache.Load(xapFile);
        }

        private void StartDownload()
        {
            Uri address = new Uri(m_data.XapName, UriKind.RelativeOrAbsolute);
            WebClient client = new WebClient();

            switch (m_data.ActionRequired)
            {
                case PackageContent.ClassFromAssembly:
                    client.OpenReadCompleted += new OpenReadCompletedEventHandler(OnCompletionGetClassFromAssembly);
                    break;
                //case PackageContent.ResourceFromAssembly:
                //    client.OpenReadCompleted += new OpenReadCompletedEventHandler(OnCompletionGetResourceFromAssembly);
                //    break;

                default:
                    return;
            }
            client.OpenReadAsync(address);
        }

        protected void Initialize(string xap, string asm, string cls, PackageContent action)
        {
            if (m_data == null)
                m_data = new DownloaderData();

            m_data.AssemblyName = asm;
            m_data.ClassName = cls;
            m_data.XapName = xap;
            m_data.ActionRequired = action;
        }

        #endregion


        #region Handlers

        private void OnCompletionGetClassFromAssembly(object sender, OpenReadCompletedEventArgs e)
        {
            // If no handler registered at the application level, why even bothering?
            if (XapDownloaded == null)
                return;

            if (e.Error != null)
                return;

            // Save to the cache
            DownloadCache.Add(m_data.XapName, e.Result);

            // Process and extract resources
            FindClassFromAssembly(e.Result);
        }

        private void FindClassFromAssembly(Stream content)
        {
            // Load a particular assembly from XAP
            Assembly a = GetAssemblyFromPackage(m_data.AssemblyName, content);

            // Get an instance of the specified user control class
            object page = a.CreateInstance(m_data.ClassName);

            // Fire the event
            XapEventArgs args = new XapEventArgs();
            args.DownloadedContent = page as UserControl;
            XapDownloaded(this, args);
        }

        private Assembly GetAssemblyFromPackage(string assemblyName, Stream xapStream)
        {
            // Local variables
            Uri assemblyUri = null;
            StreamResourceInfo resPackage = null;
            StreamResourceInfo resAssembly = null;
            AssemblyPart part = null;

            // Initialize
            assemblyUri = new Uri(assemblyName, UriKind.Relative);
            resPackage = new StreamResourceInfo(xapStream, null);
            resAssembly = Application.GetResourceStream(
                                      resPackage, assemblyUri);

            // Extract an assembly 
            part = new AssemblyPart();
            Assembly a = part.Load(resAssembly.Stream);


            // Load other assemblies (dependencies) from manifest
            Uri manifestUri = new Uri("AppManifest.xaml", UriKind.Relative);
            Stream manifestStream = Application.GetResourceStream(resPackage, manifestUri).Stream; 
            string manifest = new StreamReader(manifestStream).ReadToEnd();

            // Parse the manifest using LINQ-to-XML to get the list of referenced assemblies
            List<AssemblyPart> parts = ManifestHelper.GetDeploymentParts(manifest);

            foreach (AssemblyPart ap in parts)  
            {
                // Skip over primary assembly (done with it) and load all the others in memory
                if (!ap.Source.ToLower().Equals(assemblyName))
                {
                    StreamResourceInfo sri = null;
                    sri = Application.GetResourceStream(resPackage, new Uri(ap.Source, UriKind.Relative));
                    ap.Load(sri.Stream);
                }
            }

            // Close stream and returns
            xapStream.Close();
            return a;
        }

        #endregion    
    }
}
